@@ -1,12 +1,12 @@
 --
--- (C) 2017-18 - ntop.org
+-- (C) 2017-22 - ntop.org
 --
 
 dirs = ntop.getDirs()
 package.path = dirs.installdir .. "/scripts/lua/modules/?.lua;" .. package.path
 
 require "lua_utils"
-local host_pools_utils = require "host_pools_utils"
+local host_pools_nedge = require "host_pools_nedge"
 local discover = require "discover_utils"
 local json = require "dkjson"
 
@@ -49,28 +49,16 @@ if((ifid ~= nil) and (isAdministrator())) then
   if pool_id ~= nil then
     local active_hosts = interface.getHostsInfo(false, nil, nil, nil, nil, nil, nil, nil, nil, nil, true--[[no macs]], tonumber(pool_id)).hosts
     local network_stats = interface.getNetworksStats()
-    local pool_members = host_pools_utils.getPoolMembers(ifid, pool_id) or {}
+    local pool_members = host_pools_nedge.getPoolMembers(pool_id) or {}
 
     for _,member in ipairs(pool_members) do
       local is_mac = isMacAddress(member.address)
 
       if matches_members_filter(member.key, is_mac) then
         if (i >= start_i) and (i <= stop_i) then
-          local host_key, is_network = host_pools_utils.getMemberKey(member.key)
+          local host_key, is_network = host_pools_nedge.getMemberKey(member.key)
           local is_host = (not is_network) and (not is_mac)
           local mac_info = interface.getMacInfo(host_key)
-          local link
-
-          if active_hosts[host_key] then
-            link = ntop.getHttpPrefix() .. "/lua/host_details.lua?" .. hostinfo2url(active_hosts[host_key])
-          elseif is_mac and mac_info ~= nil then
-            link = ntop.getHttpPrefix() .. "/lua/mac_details.lua?host=" .. host_key
-          elseif is_network and network_stats[host_key] ~= nil then
-            link = ntop.getHttpPrefix() .. "/lua/hosts_stats.lua?network=" .. network_stats[host_key].network_id
-          else
-            link = ""
-          end
-
           local alias = ""
           local icon = ""
           if is_mac then
@@ -85,7 +73,7 @@ if((ifid ~= nil) and (isAdministrator())) then
               alias = ""
             end
           elseif is_host then
-            alias = getHostAltName(host_key)
+            alias = hostinfo2label(hostkey2hostinfo(host_key))
 
             if alias == host_key then
               alias = ""
@@ -93,7 +81,7 @@ if((ifid ~= nil) and (isAdministrator())) then
 
             if active_hosts[host_key] and active_hosts[host_key].mac then
               if isEmptyString(alias) then
-                alias = getHostAltName(active_hosts[host_key].mac)
+                alias = mac2label(active_hosts[host_key].mac)
                 if alias == active_hosts[host_key].mac then
                   alias = ""
                 end
@@ -114,7 +102,6 @@ if((ifid ~= nil) and (isAdministrator())) then
             column_alias = alias,
             column_icon = icon,
             column_vlan = tostring(member.vlan),
-            column_link = link,
             column_editable = tostring(tonumber(member.residual) == nil),
             column_residual = tonumber(member.residual) and secondsToTime(member.residual) or "Persistent",
             column_icon_label = discover.devtype2string(icon),
@@ -130,15 +117,15 @@ if((ifid ~= nil) and (isAdministrator())) then
   else
     local by_pool_name = {}
 
-    for _,pool in pairs(host_pools_utils.getPoolsList(ifid)) do
-      if pool.id ~= host_pools_utils.DEFAULT_POOL_ID then
+    for _,pool in pairs(host_pools_nedge.getPoolsList()) do
+      if pool.id ~= host_pools_nedge.DEFAULT_POOL_ID then
         by_pool_name[pool.name] = pool
       end
     end
 
     for _,pool in pairsByKeys(by_pool_name, asc_insensitive) do
       if (i >= start_i) and (i <= stop_i) then
-        local undeletable_pools = host_pools_utils.getUndeletablePools(ifid)
+        local undeletable_pools = host_pools_nedge.getUndeletablePools()
 
         res.data[#res.data + 1] = {
           column_pool_id = pool.id,
