@@ -1,5 +1,5 @@
 --
--- (C) 2014-15 - ntop.org
+-- (C) 2014-22 - ntop.org
 --
 
 -- This file contains the description of all functions
@@ -10,6 +10,8 @@ local prefs = ntop.getPrefs()
 local info = ntop.getInfo()
 local menu_subpages = require "prefs_menu"
 show_advanced_prefs_key = "ntopng.prefs.show_advanced_prefs"
+local have_nedge = ntop.isnEdge()
+local skip_redis = false
 
 -- ############################################
 
@@ -33,28 +35,47 @@ show_advanced_prefs_key = "ntopng.prefs.show_advanced_prefs"
 --    - hidden: (optional) if true, this subpage is hidden
 --
 
+-- ############################################
+
+DNS_PRESETS = {
+  {id="comodo_secure", label="Comodo Secure DNS", url="https://www.comodo.com/secure-dns/", primary_dns="8.26.56.26", secondary_dns="8.20.247.20"},
+  {id="dyn_internet_guide", label="Dyn Internet Guide", url="http://dyn.com/labs/dyn-internet-guide/", primary_dns="216.146.35.35", secondary_dns="216.146.36.36"},
+  --{id="fool_dns", label="FoolDNS", url="http://www.fooldns.com/fooldns-community/english-version/", primary_dns="87.118.111.215", secondary_dns="213.187.11.62"},
+  --{id="greenteam_internet", label="GreenTeam Internet", url="http://members.greentm.co.uk/", primary_dns="81.218.119.11", secondary_dns="209.88.198.133"},
+  {id="opendns", label="OpenDNS", url="https://www.opendns.com/", primary_dns="208.67.222.222", secondary_dns="208.67.220.220"},
+  {id="opendns_familyshield", label="OpenDNS - FamilyShield", url="https://www.opendns.com/setupguide/?url=familyshield", primary_dns="208.67.222.123", secondary_dns="208.67.220.123", child_safe=true},
+  {id="quad9_security", label="Quad 9: Security", url="https://quad9.net", primary_dns="9.9.9.9", secondary_dns=""},
+  {id="google", label="Google DNS", url="https://developers.google.com/speed/public-dns", primary_dns="8.8.8.8", secondary_dns="8.8.4.4"},
+  {id="cloudflare", label="Cloudflare DNS", url="https://www.cloudflare.com/learning/dns/what-is-1.1.1.1", primary_dns="1.1.1.1", secondary_dns="1.0.0.1"},
+  {id="cleanbrowsing_security", label="CleanBrowsing - Security", url="https://cleanbrowsing.org",  primary_dns="185.228.168.9", secondary_dns="185.228.169.9"},
+  {id="cleanbrwosing_adult", label="CleanBrowsing - Adult Filter", url="https://cleanbrowsing.org",  primary_dns="185.228.168.168", secondary_dns="185.228.169.168", child_safe=true},
+  {id="yandex_safe", label="Yandex - Safe", url="https://dns.yandex.com", primary_dns="77.88.8.88", secondary_dns="77.88.8.2"},
+  {id="yandex_family", label="Yandex - Family", url="https://dns.yandex.com", primary_dns="77.88.8.7", secondary_dns="77.88.8.3", child_safe=true},
+}
+
 function isSubpageAvailable(subpage, show_advanced_prefs)
   if show_advanced_prefs == nil then
     show_advanced_prefs = toboolean(ntop.getPref(show_advanced_prefs_key))
   end
-
-  if (subpage.disabled) or
+  if (subpage.hidden) or
      ((subpage.advanced) and (not show_advanced_prefs)) or
      ((subpage.pro_only) and (not ntop.isPro())) or
-     ((subpage.enterprise_only) and (not info["version.enterprise_edition"])) then
+     (subpage.enterprise_l_only and (not info["version.enterprise_l_edition"]) and (not info["version.nedge_enterprise_edition"])) or
+     ((subpage.enterprise_only) and (not info["version.enterprise_edition"]) and (not have_nedge)) or
+     (subpage.nedge_hidden) and (have_nedge) then
     return false
   end
 
   return true
 end
 
-local subpage_active = nil
-
 function prefsGetActiveSubpage(show_advanced_prefs, tab)
+  local subpage_active = nil
+
   for _, subpage in ipairs(menu_subpages) do
     if not isSubpageAvailable(subpage, show_advanced_prefs) then
-      subpage.disabled = true
-      
+      subpage.hidden = true
+
       if subpage.id == tab then
         -- will set to default
         tab = nil
@@ -80,24 +101,31 @@ function prefsGetActiveSubpage(show_advanced_prefs, tab)
 end
 
 function printMenuSubpages(tab)
+
   for _, subpage in ipairs(menu_subpages) do
-    if not subpage.disabled then
-      print[[<a href="]] print(ntop.getHttpPrefix()) print[[/lua/admin/prefs.lua?tab=]] print(subpage.id) print[[" class="list-group-item menu-item]] if(tab == subpage.id) then print(" active") end print[[">]] print(subpage.label) print[[</a>]]
+    if not subpage.hidden then
+      local url = ternary(subpage.disabled, "#", ntop.getHttpPrefix() .. [[/lua/admin/prefs.lua?tab=]] .. (subpage.id))
+      print[[<a href="]] print(url) print[[" class="list-group-item list-group-item-action]]
+
+      if(tab == subpage.id) then
+        print(" active")
+      elseif subpage.disabled then
+        print(" disabled")
+      end
+
+      print[[">]] print(subpage.label) print[[</a>]]
     end
   end
 end
 
 -- ############################################
 
--- notify ntopng upon preference changes
-function notifyNtopng(key)
+--- Notify ntopng upon preference changes
+function notifyNtopng(key, value)
     if key == nil then return end
     -- notify runtime ntopng configuration changes
-    if string.starts(key, 'nagios') then
-        if verbose then io.write('notifying ntopng upon nagios pref change\n') end
-        ntop.reloadNagiosConfig()
-    elseif string.starts(key, 'toggle_logging_level') then
-        if verbose then io.write('notifying ntopng upon logging level pref change\n') end 
+    if string.starts(key, 'toggle_logging_level') then
+        if verbose then io.write('notifying ntopng upon logging level pref change\n') end
         ntop.setLoggingLevel(value)
     end
 end
@@ -107,7 +135,7 @@ end
 local options_script_loaded = false
 local options_ctr = 0
 
-function prefsResolutionButtons(fmt, value, fixed_id)
+function prefsResolutionButtons(fmt, value, fixed_id, format_spec, max_val)
   local ctrl_id
   if fixed_id ~= nil then
     ctrl_id = fixed_id
@@ -116,7 +144,9 @@ function prefsResolutionButtons(fmt, value, fixed_id)
     options_ctr = options_ctr + 1
   end
 
-  local res = makeResolutionButtons(FMT_TO_DATA_TIME, ctrl_id, fmt, value, {classes={"pull-right"}})
+  local res = makeResolutionButtons(format_spec or FMT_TO_DATA_TIME, ctrl_id, fmt, value, {classes={"float-right"}}, max_val)
+
+  res.value = truncate(res.value)
 
   print(res.html)
   print("<script>")
@@ -143,14 +173,14 @@ function prefsInputFieldPrefs(label, comment, prekey, key, default_value, _input
     k = prekey.."."..key
   end
 
+ if not skip_redis then
   if(_POST[key] ~= nil) then
     v_s = _POST[key]
     v = tonumber(v_s)
 
     v_cache = ntop.getPref(k)
     value = v_cache
-    if ((v_cache==nil) or (v ~= v_cache)) then
-
+    if ((v_cache==nil) or (v_s ~= v_cache)) then
       if(v ~= nil and (v > 0) and (v <= 86400)) then
         ntop.setPref(k, tostring(v))
         value = v
@@ -160,6 +190,8 @@ function prefsInputFieldPrefs(label, comment, prekey, key, default_value, _input
         	v_s = string.gsub(v_s, "ldap:__", "ldap://")
 		v_s = string.gsub(v_s, "http:__", "http://")
 		v_s = string.gsub(v_s, "https:__", "https://")
+		v_s = string.gsub(v_s, "smtp:__", "smtp://")
+		v_s = string.gsub(v_s, "smtps:__", "smtps://")
 	end
         ntop.setPref(k, v_s)
         value = v_s
@@ -175,12 +207,15 @@ function prefsInputFieldPrefs(label, comment, prekey, key, default_value, _input
     value = v_s
     if((v_s==nil) or (v_s=="") or (v_s=="nil")) then
       value = default_value
-      if not isEmptyString(prekey) then
+      if not isEmptyString(prekey) and (ntop.getPref(k) ~= tostring(default_value)) then
         ntop.setPref(k, tostring(default_value))
         notifyNtopng(key)
       end
     end
   end
+ else
+   value = default_value
+ end -- skip_redis
 
   if ((showEnabled == nil) or (showEnabled == true)) then
     showEnabled = "table-row"
@@ -206,12 +241,39 @@ function prefsInputFieldPrefs(label, comment, prekey, key, default_value, _input
     end
   end
 
+  if extra.minlength ~= nil then
+    if extra.tformat ~= nil then
+      attributes["data-minlength"] = extra.minlength
+    else
+      attributes["minlength"] = extra.minlength
+    end
+  end
+
+  if extra.maxlength ~= nil then
+    if extra.tformat ~= nil then
+      attributes["data-maxlength"] = extra.maxlength
+    else
+      attributes["maxlength"] = extra.maxlength
+    end
+  end
+
+  if extra.step ~= nil then
+    if extra.tformat ~= nil then
+      attributes["data-step"] = extra.step
+    else
+      attributes["step"] = extra.step
+    end
+  end
+
   if extra.disabled == true then attributes["disabled"] = "disabled" end
   if extra.required == true then attributes["required"] = "" end
   if extra.pattern ~= nil then attributes["pattern"] = extra.pattern end
 
   if (_input_type == "number") then
     attributes["required"] = "required"
+  elseif (_input_type == "password") then
+    -- disable chrome autocomplete
+    attributes["autocomplete"] = "new-password"
   end
 
   local input_type = "text"
@@ -224,12 +286,12 @@ function prefsInputFieldPrefs(label, comment, prekey, key, default_value, _input
 
   print [[
     <td align=right>
-      <table class="form-group" style="margin-bottom: 0; min-width:22em;">
+      <table class="table table-borderless form-group mb-3" style="margin-bottom: 0; min-width:22em;">
         <tr>
           <td width="100%;"></td>
           <td style="vertical-align:top;">]]
       if extra.tformat ~= nil then
-        value = prefsResolutionButtons(extra.tformat, value)
+        value = prefsResolutionButtons(extra.tformat, value, nil, extra.format_spec, extra.max)
       end
 
       if extra.width == nil then
@@ -243,21 +305,22 @@ function prefsInputFieldPrefs(label, comment, prekey, key, default_value, _input
         style["width"] = "15em"
       end
       style["margin-left"] = "auto"
+      style["display"] = "inline"
 
       style = table.merge(style, extra.style)
       attributes = table.merge(attributes, extra.attributes)
 
       print[[
           </td>
-          <td style="vertical-align:top; padding-left: 2em;">
+          <td style="vertical-align:top; padding-left: 2em; white-space: nowrap;">
             <input id="id_input_]] print(key) print[[" type="]] print(input_type) print [[" class="form-control" ]] print(table.tconcat(attributes, "=", " ", nil, '"')) print[[ name="]] print(key) print [[" style="]] print(table.tconcat(style, ":", "; ", ";")) print[[" value="]] print(value..'"')
           if disableAutocomplete then print(" autocomplete=\"off\"") end
-        print [[/>
+        print [[/>]] print(extra.append or "") print[[
           </td>
         </tr>
         <tr>
           <td colspan="3" style="padding:0;">
-            <div class="help-block with-errors text-right" style="height:1em;"></div>
+            <div class="help-block with-errors text-end" style="height:1em;"></div>
           </td>
         </tr>
       </table>
@@ -266,6 +329,83 @@ function prefsInputFieldPrefs(label, comment, prekey, key, default_value, _input
 
 end
 
+-- ############################################
+
+function prefsDropdownFieldPrefs(label, comment, key, values, default_value, showEnabled, extra)
+  extra = extra or {}
+
+  if extra.save_pref and table.len(_POST) > 0 and isEmptyString(_POST[key]) == false and isEmptyString(extra.pref_key) == false then
+     ntop.setPref(extra.pref_key, _POST[key])
+     default_value = _POST[key]
+  end
+
+  if ((showEnabled == nil) or (showEnabled == true)) then
+    showEnabled = "table-row"
+  else
+    showEnabled = "none"
+  end
+
+  local attributes = {}
+
+  if extra.disabled == true then attributes["disabled"] = "disabled" end
+  if extra.required == true then attributes["required"] = "" end
+
+  local input_type = "text"
+
+  print('<tr id="'..key..'" style="display: '..showEnabled..';"><td width=50%><strong>'..label..'</strong><p><small>'..comment..'</small></td>')
+
+  local style = {}
+  style["text-align"] = "right"
+  style["margin-bottom"] = "0.5em"
+
+  print [[
+    <td align=right>
+      <table class="form-group mb-3" style="margin-bottom: 0; min-width:22em;">
+        <tr>
+          <td width="100%;"></td>]]
+
+      if extra.width == nil then
+	 style["width"] = "20em"
+	 style["margin-left"] = "auto"
+      else
+        style["width"] = "15em"
+      end
+      style["margin-left"] = "auto"
+
+      style = table.merge(style, extra.style)
+      attributes = table.merge(attributes, extra.attributes)
+
+      print[[
+          <td style="vertical-align:top; padding-left: 2em;">
+            <select id="id_input_]] print(key)  print [[" class="form-select" ]] print(table.tconcat(attributes, "=", " ", nil, '"')) print[[ name="]] print(key) print [[" style="]] print(table.tconcat(style, ":", "; ", ";")) print[[" value="]] print((value or '')..'"') print[[>]]
+      if extra.keys == nil then
+         for _, optname in pairs(values) do
+	 print("<option " .. ternary(optname == default_value, "selected", "") .. ">"..optname.."</option>")
+         end
+      else
+         for idx, _ in pairs(values) do
+            local key = extra.keys[idx]
+            local val = values[idx]
+            print("<option value=".. key .." " .. ternary(key == default_value, "selected", "") .. ">"..val.."</option>")
+         end
+      end
+      print[[
+            </select>
+          </td>
+        </tr>
+        <tr>
+          <td colspan="3" style="padding:0;">
+            <div class="help-block with-errors text-end" style="height:1em;"></div>
+          </td>
+        </tr>
+      </table>
+  </td></tr>
+]]
+
+end
+
+-- ############################################
+
 function prefsInformativeField(label, comment, showEnabled, extra)
   local extra = extra or {}
   extra["style"] = extra["style"] or {}
@@ -290,12 +430,12 @@ function toggleTableButton(label, comment, on_label, on_value, on_color , off_la
   -- Read it anyway to
   if(value == off_value) then
     rev_value  = on_value
-    on_active  = "btn-default"
+    on_active  = "btn-secondary"
     off_active = "btn-"..off_color.." active"
   else
     rev_value  = off_value
     on_active  = "btn-"..on_color.." active"
-    off_active = "btn-default"
+    off_active = "btn-secondary"
   end
 
   if(label ~= "") then print('<tr><td width=50%><strong>'..label..'</strong><p><small>'..comment..'</small></td><td align=right>\n') end
@@ -310,8 +450,29 @@ function toggleTableButton(label, comment, on_label, on_value, on_color , off_la
   return(value)
 end
 
-function toggleTableButtonPrefs(label, comment, on_label, on_value, on_color , off_label, off_value, off_color, submit_field,
-                                redis_key, default_value, disabled, elementToSwitch, hideOn, showElement)
+local function printNestedToSwitchLogic(nested_to_switch, submit_field)
+  if((type(nested_to_switch) ~= "table") or (table.empty(nested_to_switch))) then
+    return
+  end
+
+  for _, item in ipairs(nested_to_switch) do
+    print[[
+      if(($("#]] print(submit_field) print[[").val() == "]] print(item.pref_enabled_value or "1") print[[") &&
+         ($("#]] print(item.parent) print[[").val() == "]] print(item.parent_enabled_value) print[["))
+        $("#]] print(item.input) print[[").css("display", "table-row");
+      else
+        $("#]] print(item.input) print[[").css("display", "none");
+    ]]
+  end
+end
+
+local function toggleTableButtonPrefs(label, comment, on_label, on_value, on_color , off_label, off_value, off_color, submit_field,
+                                redis_key, default_value, disabled, elementToSwitch, hideOn, showElement, nested_to_switch)
+
+
+ local value
+
+ if not skip_redis then
   value = ntop.getPref(redis_key)
   if(_POST[submit_field] ~= nil) then
     if ( (value == nil) or (value ~= _POST[submit_field])) then
@@ -330,6 +491,9 @@ function toggleTableButtonPrefs(label, comment, on_label, on_value, on_color , o
       notifyNtopng(submit_field)
     end
   end
+ else
+   value = default_value
+ end
 
   if (disabled == true) then
     disabled = 'disabled = ""'
@@ -338,13 +502,8 @@ function toggleTableButtonPrefs(label, comment, on_label, on_value, on_color , o
   end
 
   -- Read it anyway to
-  if(value == off_value) then
-    on_active  = "btn-default"
-    off_active = "btn-"..off_color.." active"
-  else
+  if(value ~= off_value) then
     value = on_value
-    on_active  = "btn-"..on_color.." active"
-    off_active = "btn-default"
   end
 
   local objRow = ""
@@ -354,22 +513,34 @@ function toggleTableButtonPrefs(label, comment, on_label, on_value, on_color , o
     objRow = " style=\"display:table-row\""
   end
   if(label ~= "") then print('<tr id="row_'..submit_field..'"'..objRow..'><td width=50%><strong>'..label..'</strong><p><small>'..comment..'</small></td><td align=right>\n') end
-  print('<div class="btn-group btn-toggle">')
-  print('<button type="button" onclick="'..submit_field..'_functionOn()" id="'..submit_field..'_on_id" '..disabled..' class="btn btn-sm  '..on_active..'">'..on_label..'</button>')
-  print('<button type="button" onclick="'..submit_field..'_functionOff()" id="'..submit_field..'_off_id" '..disabled..' class="btn btn-sm '..off_active..'">'..off_label..'</button></div>\n')
-  print('<input type=hidden id="'..submit_field..'_input" name='..submit_field..' value="'..value..'"/>\n')
-  if(label ~= "") then print('</td></tr>') end
-  print('\n')
-  print('<script>\n')
-  print[[function ]] print(submit_field) print [[_functionOn(){
-    var classOn = document.getElementById("]] print(submit_field) print [[_on_id");
-    var classOff = document.getElementById("]] print(submit_field) print [[_off_id");
-    classOn.removeAttribute("class");
-    classOff.removeAttribute("class");
-    classOn.setAttribute("class", "btn btn-sm btn-]]print(on_color) print[[ active");
-    classOff.setAttribute("class", "btn btn-sm btn-default");
 
-    $("#]] print(submit_field) print [[_input").val("]] print(on_value) print[[").trigger('change');]]
+  print([[
+    <div class="form-switch">
+      <input ]].. (value == off_value and '' or 'checked') ..[[ type="checkbox" class="form-check-input" id="check-]].. submit_field ..[[">
+      <label class="form-check-label" for="check-]].. submit_field ..[["></label>
+      <input hidden id="input-]].. submit_field ..[[" name="]].. submit_field ..[[" value="]].. value ..[[">
+  ]])
+  print([[</div>]])
+  if(label ~= "") then print('</td></tr>') end
+  print('<script type="text/javascript">')
+
+  print([[
+    $("#check-]].. submit_field.. [[").change(function(e) {
+
+      const value = $(this).is(":checked");
+
+      if (value) {
+        ]]..submit_field..[[_functionOn();
+      }
+      else {
+        ]]..submit_field..[[_functionOff();
+      }
+    });
+  ]])
+
+  print[[function ]] print(submit_field) print [[_functionOn(){]]
+    print([[$(`#input-]].. submit_field ..[[`).val("]].. on_value ..[[");]])
+
     if elementToSwitch ~= nil then
       for element = 1, #elementToSwitch do
         if ((hideOn == nil) or (hideOn == false)) then
@@ -379,18 +550,17 @@ function toggleTableButtonPrefs(label, comment, on_label, on_value, on_color , o
         end
       end
     end
+
+    printNestedToSwitchLogic(nested_to_switch, "input-" .. submit_field)
+
     print[[
   }
   ]]
   print[[
-  function ]] print(submit_field) print [[_functionOff(){
-    var classOn = document.getElementById("]] print(submit_field) print [[_on_id");
-    var classOff = document.getElementById("]] print(submit_field) print [[_off_id");
-    classOn.removeAttribute("class");
-    classOff.removeAttribute("class");
-    classOn.setAttribute("class", "btn btn-sm btn-default");
-    classOff.setAttribute("class", "btn btn-sm btn-]]print(off_color) print[[ active");
-    $("#]] print(submit_field) print [[_input").val("]]print(off_value) print[[").trigger('change');]]
+  function ]] print(submit_field) print [[_functionOff(){]]
+
+    print([[$(`#input-]].. submit_field ..[[`).val("]].. off_value ..[["); ]])
+
     if elementToSwitch ~= nil then
       for element = 1, #elementToSwitch do
         if ((hideOn == nil) or (hideOn == false)) then
@@ -400,6 +570,9 @@ function toggleTableButtonPrefs(label, comment, on_label, on_value, on_color , o
         end
       end
     end
+
+    printNestedToSwitchLogic(nested_to_switch, "input-" .. submit_field)
+
     print [[
   }]]
   print('</script>\n')
@@ -407,11 +580,12 @@ function toggleTableButtonPrefs(label, comment, on_label, on_value, on_color , o
 end
 
 local function get_pref_redis_key(options)
-  return "ntopng.prefs." .. ternary(options.pref ~= nil, options.pref, options.field)
+  local prefix = options.redis_prefix or "ntopng.prefs."
+  return prefix .. ternary(options.pref ~= nil, options.pref, options.field)
 end
 
-function prefsToggleButton(params)
-  defaults = {
+function prefsToggleButton(subpage_active, params)
+  local defaults = {
     to_switch = {},             -- a list of elements to be switched on or off
     on_text = "On",             -- The text when the button is on
     on_value = "1",             -- The value when the button is on
@@ -419,40 +593,61 @@ function prefsToggleButton(params)
     off_text = "Off",           -- The text when the button is off
     off_value = "0",            -- The value when the button is off
     off_class = "danger",       -- The css class when the button is off
-    reverse_switch = false      -- If true, elements are hidden when the item is enabled
+    reverse_switch = false,     -- If true, elements are hidden when the item is enabled
+    nested_to_switch = {},      -- Similar to "to_switch" but for nested items
   }
 
   local options = table.merge(defaults, params)
   local redis_key = get_pref_redis_key(options)
 
-  return toggleTableButtonPrefs(subpage_active.entries[options.field].title, subpage_active.entries[options.field].description .. (subpage_active.entries[options.field].content or ""),
+  -- used for debugging purpose:
+  -- if (subpage_active.entries[options.field] ~= nil) then
+  --  traceError(TRACE_ERROR, TRACE_CONSOLE, "Missing entry '" .. options.field .. "' inside entries field")
+  -- end
+
+  return toggleTableButtonPrefs(params.title or subpage_active.entries[options.field].title,
+    (params.description or subpage_active.entries[options.field].description) .. (params.content or subpage_active.entries[options.field].content or ""),
     options.on_text, options.on_value, options.on_class,
     options.off_text, options.off_value, options.off_class,
     options.field, redis_key,
-    options.default, options.disabled, options.to_switch, options.reverse_switch, not options.hidden)
+    options.default, options.disabled, options.to_switch, options.reverse_switch, not options.hidden, options.nested_to_switch)
 end
 
 function multipleTableButtonPrefs(label, comment, array_labels, array_values, default_value, selected_color,
                                   submit_field, redis_key, disabled, elementToSwitch, showElementArray,
-                                  javascriptAfterSwitch, showElement)
-  if(_POST[submit_field] ~= nil) then
-    ntop.setPref(redis_key, _POST[submit_field])
+                                  nested_to_switch, showElement, initialValue, toggleElementArray)
+  local value
+  local disabled_attr
+
+  if not skip_redis then
+   if(_POST[submit_field] ~= nil) then
+    local old_v = ntop.getPref(redis_key)
     value = _POST[submit_field]
-    notifyNtopng(submit_field)
-  else
-    value = ntop.getPref(redis_key)
+
+    if old_v ~= _POST[submit_field] then
+      ntop.setPref(redis_key, _POST[submit_field])
+      notifyNtopng(submit_field)
+    end
+   else
+    value = initialValue or ntop.getPref(redis_key)
+
     if(value == "") then
       if(default_value ~= nil) then
         ntop.setPref(redis_key, default_value)
         value = default_value
       end
     end
+   end
+  else
+    value = default_value
   end
 
   if (disabled == true) then
-    disabled = 'disabled = ""'
+    disabled = "disabled"
+    disabled_attr = ' disabled="disabled"'
   else
     disabled = ""
+    disabled_attr = ""
   end
 
   local objRow = ""
@@ -463,10 +658,10 @@ function multipleTableButtonPrefs(label, comment, array_labels, array_values, de
   end
   if(value ~= nil) then
     if(label ~= "") then print('<tr id="row_'..submit_field..'"'..objRow..'><td width=50%><strong>'..label..'</strong><p><small>'..comment..'</small></td><td align=right>\n') end
-    print('<div class="btn-group" data-toggle="buttons-radio" data-toggle-name="'..submit_field..'">')
+    print('<div class="btn-group" data-bs-toggle="buttons-radio" data-bs-toggle-name="'..submit_field..'">')
 
     for nameCount = 1, #array_labels do
-      local type_button = "btn-default"
+      local type_button = "btn-secondary"
       if(value == array_values[nameCount]) then
         local color
         if type(selected_color) == "table" then
@@ -476,11 +671,23 @@ function multipleTableButtonPrefs(label, comment, array_labels, array_values, de
         end
         type_button = "btn-"..color.."  active"
       end
-      print('<button id="id_'..submit_field..'_'..array_values[nameCount]..'" value="'..array_values[nameCount]..'" type="button" class="btn btn-sm '..type_button..'" data-toggle="button">'..array_labels[nameCount]..'</button>\n')
+      print('<button id="id_'..submit_field..'_'..array_values[nameCount]..'" value="'..array_values[nameCount]..'" type="button" class="btn btn-sm '..type_button..' ' .. disabled .. '"'.. disabled_attr ..' data-bs-toggle="button">'..array_labels[nameCount]..'</button>\n')
     end
     print('</div>\n')
     print('<input type="hidden" id="id-toggle-'..submit_field..'" name="'..submit_field..'" value="'..value..'" />\n')
     print('<script>\n')
+
+    -- showElementArray can be either:
+    --  1. a simple table (e.g. {true, false}): it affects all the elements in elementToSwitch
+    --  2. a nested table (e.g. {{true, false}, {false, false}}): each element in the table
+    --     affects the corresponding element in the elementToSwitch (e.g. {true, false} affects
+    --     elementToSwitch[1], whereas {false, false} affects elementToSwitch[2])
+    if(showElementArray and (type(showElementArray[1]) ~= "table")) then
+      -- This is simple table (case 1.) above, convert it to case 2 to handle
+      -- both cases consinstenly below.
+      showElementArray = {showElementArray, }
+    end
+
     for nameCount = 1, #array_labels do
       print('$("#id_'..submit_field..'_'..array_values[nameCount]..'").click(function() {\n')
       print(' var field = $(\'#id-toggle-'..submit_field..'\');\n')
@@ -500,29 +707,51 @@ function multipleTableButtonPrefs(label, comment, array_labels, array_values, de
         if(array_values[indexLabel] == array_values[nameCount]) then
           print[[class_]] print(array_values[indexLabel]) print[[.setAttribute("class", "btn btn-sm btn-]]print(color) print[[ active");]]
         else
-          print[[class_]] print(array_values[indexLabel]) print[[.setAttribute("class", "btn btn-sm btn-default");]]
+          print[[class_]] print(array_values[indexLabel]) print[[.setAttribute("class", "btn btn-sm btn-secondary");]]
         end
       end
 
+      -- Show/Hide all the elementToSwitch items at once
       if (showElementArray ~= nil) then
-      for indexSwitch = 1, #showElementArray do
-        if (indexSwitch == nameCount) then
-          if elementToSwitch ~= nil then
-            for element = 1, #elementToSwitch do
-              if (showElementArray[indexSwitch] == true) then
-                print('$("#'..elementToSwitch[element]..'").css("display","table-row");\n')
-              else
-                print('$("#'..elementToSwitch[element]..'").css("display","none");\n')
+        for indexSwitch = 1, #showElementArray[1] do
+          if (indexSwitch == nameCount) then
+            if elementToSwitch ~= nil then
+              for element = 1, #elementToSwitch do
+                -- Try to get the element-specific configuration if available,
+                -- otherwise fallback to the default configuration (at index 1)
+                local to_apply = showElementArray[element] or showElementArray[1]
+
+                if (to_apply[indexSwitch] == true) then
+                  -- NOTE: this is executed into the js change callback
+                  print('$("#'..elementToSwitch[element]..'").css("display","table-row");\n')
+                else
+                  -- NOTE: this is executed into the js change callback
+                  print('$("#'..elementToSwitch[element]..'").css("display","none");\n')
+                end
+              end
+            end
+          end
+        end
+      -- Show/Hide all the elementToSwitch selectively
+      elseif (toggleElementArray ~= nil) then
+        for indexSwitch = 1, #toggleElementArray do
+          if (indexSwitch == nameCount) then
+            if elementToSwitch ~= nil then
+              for element = 1, #elementToSwitch do
+                if (toggleElementArray[indexSwitch][element] == true) then
+                  -- NOTE: this is executed into the js change callback
+                  print('$("#'..elementToSwitch[element]..'").css("display","table-row");\n')
+                else
+                  -- NOTE: this is executed into the js change callback
+                  print('$("#'..elementToSwitch[element]..'").css("display","none");\n')
+                end
               end
             end
           end
         end
       end
-      end
 
-      if javascriptAfterSwitch ~= nil then
-        print(javascriptAfterSwitch)
-      end
+      printNestedToSwitchLogic(nested_to_switch, "id-toggle-" .. submit_field)
 
       print('});\n')
     end
@@ -550,9 +779,9 @@ function loggingSelector(label, comment, submit_field, redis_key)
   end
 
   local logging_values = {"trace", "debug", "info", "normal", "warning", "error"}
-  local color_map = {"default", "success", "info", "primary", "warning", "danger"}
+  local color_map = {"secondary", "success", "info", "primary", "warning", "danger"}
   local logging_keys = {}
-  local color = "default"
+  local color = "secondary"
 
   for i,v in ipairs(logging_values) do logging_keys[i] = firstToUpper(v) end
 
@@ -561,3 +790,15 @@ function loggingSelector(label, comment, submit_field, redis_key)
 
   return(value)
 end
+
+function printPageSection(section_name)
+   print('<thead class="table-primary"><tr><th colspan=2 class="info">'..section_name..'</th></tr></thead>')
+end
+
+function printSaveButton(some_content)
+  print('<tr><td colspan=2 style="text-align:right;">' .. (some_content or "") .. '<button type="submit" class="btn btn-primary" style="width:115px" disabled="disabled">'..i18n("save")..'</button></td></tr>')
+end
+
+function prefsSkipRedis(skip)
+  skip_redis = skip
+end
