@@ -1,6 +1,6 @@
 /*
  *
- * (C) 2013-17 - ntop.org
+ * (C) 2013-22 - ntop.org
  *
  *
  * This program is free software; you can redistribute it and/or modify
@@ -21,18 +21,20 @@
 
 #include "ntop_includes.h"
 
+/* ******************************************* */
 
-StatsManager::StatsManager(int interface_id, const char *filename) : StoreManager(interface_id) {
-  char filePath[MAX_PATH], fileFullPath[MAX_PATH], fileName[MAX_PATH];
+StatsManager::StatsManager(int interface_id, const char *filename) : SQLiteStoreManager(interface_id) {
+  char filePath[MAX_PATH+16], fileFullPath[MAX_PATH+162], fileName[MAX_PATH+16];
 
   MINUTE_CACHE_NAME = "MINUTE_STATS";
   HOUR_CACHE_NAME = "HOUR_STATS";
   DAY_CACHE_NAME = "DAY_STATS";
 
-  snprintf(filePath, sizeof(filePath), "%s/%d/top_talkers/",
+  snprintf(filePath, sizeof(filePath)-1, "%s/%d/top_talkers/",
            ntop->get_working_dir(), ifid);
   strncpy(fileName, filename, sizeof(fileName));
-  snprintf(fileFullPath, sizeof(fileFullPath), "%s/%d/top_talkers/%s",
+  fileName[sizeof(fileName) - 1] = '\0';
+  snprintf(fileFullPath, sizeof(fileFullPath)-1, "%s/%d/top_talkers/%s",
 	   ntop->get_working_dir(), ifid, filename);
   ntop->fixPath(filePath);
   ntop->fixPath(fileFullPath);
@@ -46,6 +48,8 @@ StatsManager::StatsManager(int interface_id, const char *filename) : StoreManage
   init(fileFullPath);
 }
 
+/* ******************************************* */
+
 /**
  * @brief Opens a new cache to be used to store statistics.
  * @brief This function implements opening a new cache to store stats
@@ -84,6 +88,8 @@ int StatsManager::openStore(const char *cache_name)
   return rc;
 }
 
+/* ******************************************* */
+
 /**
  * @brief Database interface to implement stats purging.
  * @details This function implements the database-specific code
@@ -118,6 +124,8 @@ int StatsManager::deleteStatsOlderThan(const char * const cache_name, const time
   return rc;
 }
 
+/* ******************************************* */
+
 /**
  * @brief Minute stats interface to database purging.
  * @details This function hides cache-specific details (e.g. building the key
@@ -136,6 +144,8 @@ int StatsManager::deleteMinuteStatsOlderThan(unsigned num_days) {
   return deleteStatsOlderThan(MINUTE_CACHE_NAME, rawtime);
 }
 
+/* ******************************************* */
+
 /**
  * @brief Hour stats interface to database purging.
  * @details This function hides cache-specific details (e.g. building the key
@@ -154,6 +164,8 @@ int StatsManager::deleteHourStatsOlderThan(unsigned num_days) {
   return deleteStatsOlderThan(HOUR_CACHE_NAME, rawtime);
 }
 
+/* ******************************************* */
+
 /**
  * @brief Day stats interface to database purging.
  * @details This function hides cache-specific details (e.g. building the key
@@ -172,6 +184,8 @@ int StatsManager::deleteDayStatsOlderThan(unsigned num_days) {
   return deleteStatsOlderThan(DAY_CACHE_NAME, rawtime);
 }
 
+/* ******************************************* */
+
 /**
  * @brief Callback for completion of retrieval of an interval of stats
  *
@@ -194,6 +208,8 @@ static int get_samplings_db(void *data, int argc,
   return 0;
 }
 
+/* ******************************************* */
+
 /**
  * @brief Retrieve an interval of samplings from a database
  * @details This function implements the database-specific code
@@ -238,6 +254,8 @@ int StatsManager::retrieveStatsInterval(struct statsManagerRetrieval *retvals,
   return rc;
 }
 
+/* ******************************************* */
+
 /**
  * @brief Retrieve an interval of samplings from the minute stats cache
  * @details This function implements the database-specific code
@@ -260,6 +278,8 @@ int StatsManager::retrieveMinuteStatsInterval(time_t epoch_start,
   return retrieveStatsInterval(retvals, MINUTE_CACHE_NAME, epoch_start, epoch_end);
 }
 
+/* ******************************************* */
+
 /**
  * @brief Retrieve an interval of samplings from the hour stats cache
  * @details This function implements the database-specific code
@@ -282,6 +302,8 @@ int StatsManager::retrieveHourStatsInterval(time_t epoch_start,
   return retrieveStatsInterval(retvals, HOUR_CACHE_NAME, epoch_start, epoch_end);
 }
 
+/* ******************************************* */
+
 /**
  * @brief Retrieve an interval of samplings from the day stats cache
  * @details This function implements the database-specific code
@@ -304,6 +326,8 @@ int StatsManager::retrieveDayStatsInterval(time_t epoch_start,
   return retrieveStatsInterval(retvals, DAY_CACHE_NAME, epoch_start, epoch_end);
 }
 
+/* ******************************************* */
+
 /**
  * @brief Database interface to add a new stats sampling
  * @details This function implements the database-specific layer for
@@ -331,19 +355,16 @@ int StatsManager::insertSampling(const char * const sampling, const char * const
 
   m.lock(__FILE__, __LINE__);
 
-  if(sqlite3_prepare(db, query, -1, &stmt, 0) ||
+  if(sqlite3_prepare_v2(db, query, -1, &stmt, 0) ||
      sqlite3_bind_int64(stmt, 1, key) ||
      sqlite3_bind_text(stmt, 2, sampling, strlen(sampling), SQLITE_TRANSIENT)) {
     rc = 1;
     goto out;
   }
 
-  while((rc = sqlite3_step(stmt)) != SQLITE_DONE) {
-    if(rc == SQLITE_ERROR) {
-      ntop->getTrace()->traceEvent(TRACE_INFO, "SQL Error: step");
-      rc = 1;
-      goto out;
-    }
+  if((rc = exec_statement(stmt)) != SQLITE_DONE) {
+    rc = 1;
+    goto out;
   }
 
   rc = 0;
@@ -354,6 +375,8 @@ int StatsManager::insertSampling(const char * const sampling, const char * const
   return rc;
 }
 
+/* ******************************************* */
+
 /**
  * @brief Interface function for insertion of a new minute stats sampling
  * @details This public method implements insertion of a new sampling,
@@ -371,6 +394,8 @@ int StatsManager::insertMinuteSampling(time_t epoch, const char * const sampling
   return insertSampling(sampling, MINUTE_CACHE_NAME, static_cast<long int>(epoch));
 }
 
+/* ******************************************* */
+
 /**
  * @brief Interface function for insertion of a new hour stats sampling
  * @details This public method implements insertion of a new sampling,
@@ -388,6 +413,8 @@ int StatsManager::insertHourSampling(time_t epoch, const char * const sampling)
   return insertSampling(sampling, HOUR_CACHE_NAME, static_cast<long int>(epoch));
 }
 
+/* ******************************************* */
+
 /**
  * @brief Interface function for insertion of a new day stats sampling
  * @details This public method implements insertion of a new sampling,
@@ -430,6 +457,8 @@ static int get_sampling_db_callback(void *data, int argc,
   return 0;
 }
 
+/* ******************************************* */
+
 /**
  * @brief Database interface to retrieve a stats sampling
  * @details This function implements the database-specific layer for
@@ -470,6 +499,8 @@ int StatsManager::getSampling(string * sampling, const char * const cache_name,
   return rc;
 }
 
+/* ******************************************* */
+
 /**
  * @brief Interface function for retrieval of a minute stats sampling
  * @details This public method implements retrieval of an existing
@@ -491,3 +522,5 @@ int StatsManager::getMinuteSampling(time_t epoch, string * sampling) {
   return getSampling(sampling, MINUTE_CACHE_NAME, epoch_start, epoch_end);
 }
 
+
+/* ******************************************* */
