@@ -1,6 +1,6 @@
 /*
  *
- * (C) 2013-18 - ntop.org
+ * (C) 2013-22 - ntop.org
  *
  *
  * This program is free software; you can redistribute it and/or modify
@@ -24,7 +24,7 @@
 /* *************************************** */
 
 TcpPacketStats::TcpPacketStats() {
-  pktRetr = pktOOO = pktLost = 0;
+  pktRetr = pktOOO = pktLost = pktKeepAlive = 0;
 }
 
 /* *************************************** */
@@ -49,6 +49,7 @@ void TcpPacketStats::deserialize(json_object *o) {
   if(json_object_object_get_ex(o, "retransmissions", &obj)) pktRetr = json_object_get_int(obj); else pktRetr = 0;
   if(json_object_object_get_ex(o, "out_of_order", &obj))    pktOOO = json_object_get_int(obj);  else pktOOO = 0;
   if(json_object_object_get_ex(o, "lost", &obj))            pktLost = json_object_get_int(obj); else pktLost = 0;
+  if(json_object_object_get_ex(o, "keep_alive", &obj))      pktKeepAlive = json_object_get_int(obj); else pktKeepAlive = 0;
 }
 
 /* ******************************************* */
@@ -58,9 +59,10 @@ json_object* TcpPacketStats::getJSONObject() {
 
   my_object = json_object_new_object();
 
-  if(pktRetr > 0) json_object_object_add(my_object, "retransmissions", json_object_new_int(pktRetr));
-  if(pktOOO > 0) json_object_object_add(my_object, "out_of_order", json_object_new_int(pktOOO));
-  if(pktLost > 0) json_object_object_add(my_object, "lost", json_object_new_int(pktLost));
+  if(pktRetr)      json_object_object_add(my_object, "retransmissions", json_object_new_int64(pktRetr));
+  if(pktOOO)       json_object_object_add(my_object, "out_of_order", json_object_new_int64(pktOOO));
+  if(pktLost)      json_object_object_add(my_object, "lost", json_object_new_int64(pktLost));
+  if(pktKeepAlive) json_object_object_add(my_object, "keep_alive", json_object_new_int64(pktKeepAlive));
   
   return(my_object);
 }
@@ -73,6 +75,7 @@ void TcpPacketStats::lua(lua_State* vm, const char *label) {
   lua_push_uint64_table_entry(vm, "retransmissions", pktRetr);
   lua_push_uint64_table_entry(vm, "out_of_order", pktOOO);
   lua_push_uint64_table_entry(vm, "lost", pktLost);
+  lua_push_uint64_table_entry(vm, "keep_alive", pktKeepAlive);
   
   lua_pushstring(vm, label);
   lua_insert(vm, -2);
